import os

PROVIDER = os.environ.get('CLOUD_SERVICES_PROVIDER')

# AWS Config
AWS_REGION = os.environ.get('CLOUD_SERVICES_AWS_REGION')
AWS_ACCESS_KEY = os.environ.get('AWS_ACCESS_KEY_ID')
AWS_SECRET_KEY = os.environ.get('AWS_SECRET_ACCESS_KEY')

# Notification Bus services
NOTIFICATION = 'Notification'
SUSCRIPTION = 'Subscription'



from .. import config
from ..provider.factory import factory


def listener(func):
    """ Endpoint of Suscription and Reception of Notifications

    Implementation of `listener` of notifications bus
    for Flask. Already return response.
    The decorated function must not return anything.
    """

    def wrapper(*args, **kwargs):
        provider = factory('BusNotifications')
        result = provider.listener()
        if result.message_type == config.NOTIFICATION:
            func(data_listener=result, *args, **kwargs)
        return result.response
    wrapper.__doc__ = func.__doc__
    return wrapper


def publisher(
    topic_arn: str = None,
    message: str = None,
    *args,
    **kwargs
):
    """ Function to publication of Notifications

    Implementation of `publisher` of notifications bus

    Args:
        topic_arn (str): topic arn of channel
        message (str): message to publish
    """
    provider = factory('BusNotifications')
    return provider.publisher(
        topic_arn=topic_arn,
        message=message,
        *args,
        **kwargs
    )



import json
import logging
import requests
import boto3
from flask import request
from .. import config
from ..constants import responses as rsp
from ..config import NOTIFICATION, SUSCRIPTION
from ..contract import (
    BusNotificationsContract,
    StorageContract,
)
from ..contract import DataListener


__all__ = (
    'BusNotifications',
    'Storage',
)


class BusNotifications(BusNotificationsContract):

    def listener(self) -> DataListener:
        """Endpoint of Suscription and Reception of Notifications of AWS sns.
        Implementation of `listener` of AWS notifications bus
        for flask.

        Returns:
            ..contract.DataListener:
        """
        try:
            data = json.loads(request.data)
        except Exception as e:
            logging.error(str(e))
            return DataListener(
                response=(rsp.AWS_BAD_REQUEST, 400)
            )
        message_type = request.headers.get('X-Amz-Sns-Message-Type')
        topic_arn = request.headers.get('X-Amz-Sns-Topic-Arn')
        if message_type == 'Notification':
            # Receive notification from AWS channel and call decorated function
            result = DataListener(
                message_type=NOTIFICATION,
                message=data['Message'],
                topic_arn=topic_arn,
                response=rsp.AWS_OK,
            )
        elif (message_type == 'SubscriptionConfirmation'
                and 'SubscribeURL' in data):
            # Confirm suscription of AWS channel
            requests.get(data['SubscribeURL'])
            result = DataListener(
                message_type=SUSCRIPTION,
                topic_arn=topic_arn,
                response=rsp.AWS_OK
            )
        else:
            result = DataListener(
                response=(rsp.AWS_BAD_REQUEST, 400)
            )
        return result

    def publisher(
        self,
        topic_arn: str = None,
        message: str = None
    ) -> dict:
        """Method to publish in sns of AWS.
        Implementation of `publisher` to AWS notifications bus
        for Flask.

        Args:
            topic_arn (str)
            message (str)

        Returns:
            dict: Response returned by client sns
        """
        client = boto3.client(
            'sns',
            region_name=config.AWS_REGION,
            aws_access_key_id=config.AWS_ACCESS_KEY,
            aws_secret_access_key=config.AWS_SECRET_KEY,
        )
        # Publish a simple message to the specified SNS topic
        return client.publish(
            TopicArn=topic_arn,
            Message=message
        )


class Storage(StorageContract):

    CLIENT = boto3.client(
        's3',
        region_name=config.AWS_REGION,
        aws_access_key_id=config.AWS_ACCESS_KEY,
        aws_secret_access_key=config.AWS_SECRET_KEY,
    )

    def upload_from_filename(
        self,
        filename: str,
        bucket_name: str,
        key: str,
        extra_args: str = {},
        *args,
        **kwargs,
    ) -> None:
        """Function to upload file from given filename to S3

        Implementation of `upload_from_filename` using S3

        Args:
            filename (str): The path to the file to upload.
            bucket_name (str): The name of the bucket to upload to.
            key (str): The file name in Cloud Storage. Default None
            extra_args (dict): Extra args. Default {}
            args
            kwargs

        Note: It accept any parameter the adapter needs,
                wich does not deliver in request
        """

        return self.CLIENT.upload_file(
            Filename=filename,
            Bucket=bucket_name,
            Key=key,
            ExtraArgs=extra_args,
            *args,
            **kwargs
        )

    def upload_from_file_obj(
        self,
        file_obj: str,
        bucket_name: str,
        key: str,
        extra_args: str = {},
        *args,
        **kwargs,
    ) -> None:
        """Function to upload file from given file object to S3

        Implementation of `upload_fileobj` using S3

        Args:
            file_obj (Object): The file object to upload by S3 requirements.
            bucket_name (str): The name of the bucket to upload to.
            key (str): The file name in Cloud Storage. Default None
            extra_args (dict): Extra args. Default {}
            args
            kwargs

        Note: It accept any parameter the adapter needs,
                wich does not deliver in request
        """

        return self.CLIENT.upload_fileobj(
            Fileobj=file_obj,
            Bucket=bucket_name,
            Key=key,
            ExtraArgs=extra_args,
            *args,
            **kwargs
        )

    def download_from_filename(
        self,
        filename: str,
        bucket_name: str,
        key: str,
        extra_args: str = {},
        *args,
        **kwargs,
    ) -> None:
        """Function to download file from given filename to S3

        Implementation of `download_file` using S3

        Args:
            filename (str): The path to the file to upload.
            bucket_name (str): The name of the bucket to upload to.
            key (str): The file name in Cloud Storage.
            args
            kwargs

        Note: It accept any parameter the adapter needs,
            wich does not deliver in request
        """
        return self.CLIENT.download_file(
            Bucket=bucket_name,
            Key=key,
            Filename=filename,
            ExtraArgs=extra_args,
            *args,
            **kwargs
        )

    def get_file_url(
        self,
        bucket_name: str,
        key: str,
        *args,
        **kwargs,
    ) -> str:
        """Function to get file url

        Implementation of `get_file_url` using S3

        Args:
            bucket_name (str): The name of the bucket to upload to.
            key (str): The file name in Cloud Storage.
            args
            kwargs

        Note: It accept any parameter the adapter needs,
            wich does not deliver in request

        Returns:
            str: URL built
        """
        bucket_location = self.CLIENT.get_bucket_location(Bucket=bucket_name)
        bucket_location = bucket_location['LocationConstraint']
        _ = f"https://s3-{bucket_location}.amazonaws.com/{bucket_name}/{key}"
        return _



from .services.notifications import (
    listener as notifications_listener,
    publisher as notifications_publisher,
)
from .services.storage import (
    upload_from_filename as storage_upload_from_filename,
    upload_from_file_obj as storage_upload_from_file_obj,
    download_from_filename as storage_download_from_filename,
    get_file_url as storage_get_file_url,
)

__all__ = (
    'notifications_listener',
    'notifications_publisher',
    'storage_upload_from_filename',
    'storage_upload_from_file_obj',
    'storage_download_from_filename',
    'storage_get_file_url',
)



from ..provider.factory import factory


def upload_from_filename(
    filename: str,
    bucket_name: str,
    *args,
    **kwargs,
):
    """Function to upload file from given filename to cloud storage

    Implementation of `upload_from_filename` of Storage service

    Args:
        filename (str): The path to the file to upload.
        bucket_name (str): The name of the bucket to upload to.
        args
        kwargs

    Note: It accept any parameter the adapter needs,
        wich does not deliver in request
    """

    return factory('Storage').upload_from_filename(
        filename=filename,
        bucket_name=bucket_name,
        *args,
        **kwargs
    )


def upload_from_file_obj(
    file_obj,
    bucket_name: str,
    key: str,
    *args,
    **kwargs,
) -> None:
    """Function to upload file from given file object to cloud

    Args:
        file_obj (Object): The file object to upload by provider requirements.
        bucket_name (str): The name of the bucket to upload to.
        key (str): The file name in Cloud Storage. Default None
        extra_args (dict): Extra args. Default {}
        args
        kwargs

    Note: It accept any parameter the adapter needs,
            wich does not deliver in request
    """

    return factory('Storage').upload_from_file_obj(
        file_obj=file_obj,
        bucket_name=bucket_name,
        key=key,
        *args,
        **kwargs
    )


def download_from_filename(
    filename: str,
    bucket_name: str,
    key: str,
    *args,
    **kwargs,
):
    """Function to upload file from given filename to cloud storage

    Args:
        filename (str): The path to the file to download.
        bucket_name (str): The name of the bucket to download to.
        key (str): The file name to download to.
        args
        kwargs

    Note: It accept any parameter the adapter needs,
        wich does not deliver in request
    """

    return factory('Storage').download_from_filename(
        filename=filename,
        bucket_name=bucket_name,
        key=key,
        *args,
        **kwargs
    )


def get_file_url(
    bucket_name: str,
    key: str,
    *args,
    **kwargs,
) -> str:
    """Function to get file url from cloud storage

    Implementation of `get_file_url` from cloud storage

    Args:
        bucket_name (str): The name of the bucket to upload to.
        key (str): The file name in Cloud Storage.
        args
        kwargs

    Note: It accept any parameter the adapter needs,
        wich does not deliver in request

    Returns:
        str: URL file
    """
    return factory('Storage').get_file_url(
        bucket_name=bucket_name,
        key=key,
        *args,
        **kwargs
    )



import abc


class DataListener:
    """Object used for BusNotifications.listener()

    Attributes:
        message: type unknown
        response: type unknown
    """

    def __init__(
        self,
        message_type: str = None,
        topic_arn: str = None,
        message=None,
        response=None
    ):
        self.message_type = message_type
        self.topic_arn = topic_arn
        self.message = message
        self.response = response


class BusNotificationsContract(metaclass=abc.ABCMeta):
    """Contract for Notification service"""

    @abc.abstractmethod
    def listener(self) -> DataListener:
        """It must Subscribe or Receive Notifications from Notifications Bus
        Implementation of `listener` of Notifications Bus

        Returns:
            DataListener:
        """
        pass

    @abc.abstractmethod
    def publisher(
        self,
        topic_arn: str,
        message: str,
        *args,
        **kwargs,
    ):
        """It must Publish to Notifications Bus

        Args:
            topic_arn (str): topic arn of channel
            message (str): message to publish

        Note: It accepts any parameter the adapter needs,
            wich does not deliver in request
        """
        pass


class StorageContract(metaclass=abc.ABCMeta):
    """Contract for Storage service"""

    @abc.abstractmethod
    def upload_from_filename(
        self,
        filename: str,
        bucket_name: str,
        *args,
        **kwargs,
    ):
        """It upload a file from file-like path.

        Args:

            filename (str): The path to the file to upload.
            bucket_name (str): The name of the bucket to upload to.
            args
            kwargs

        Note: It accepts any parameter the adapter needs,
            wich does not deliver in request
        """
        pass

    @abc.abstractmethod
    def upload_from_file_obj(
        self,
        file_obj,
        bucket_name: str,
        *args,
        **kwargs,
    ):
        """It upload a file from file-like object.

        Args:

            file_obj (Object): The file object to upload by
                provider requirements.
            bucket_name (str): The name of the bucket to upload to.
            args
            kwargs

        Note: It accepts any parameter the adapter needs,
            wich does not deliver in request
        """
        pass

    @abc.abstractmethod
    def download_from_filename(
        self,
        filename: str,
        bucket_name: str,
        key: str,
        *args,
        **kwargs,
    ):
        """Function to download file from given filename.

        Args:
            filename (str): The path to the file to download to.
            bucket_name (str): The name of the bucket to download to.
            key (str): The file name in Cloud Storage.
            args
            kwargs

        Note: It accepts any parameter the adapter needs,
                wich does not deliver in request
        """
        pass

    @abc.abstractmethod
    def get_file_url(
        self,
        bucket_name: str,
        key: str,
        *args,
        **kwargs,
    ):
        """Function to get file url from given key.

        Args:
            bucket_name (str): The name of the bucket.
            key (str): The file name in Cloud Storage.
            args
            kwargs

        Note: It accepts any parameter the adapter needs,
            wich does not deliver in request
        """
        pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="flask_cloud_services",
    version="0.1.0",
    author="Terminus",
    author_email="david.tabla@zinobe.com",
    description="Cloud Services in Flask",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/terminus-zinobe/flask-cloud-services",
    packages=setuptools.find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests"]
    ),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'boto3>=1.12.40',
        'flask>=1.1.1',
        'requests>=2.23.0',
    ]
)



from .. import config
from . import aws

providers = {
    'aws': aws
}


def factory(service: str):
    """Call the provider available selected from config.PROVIDER

    Returns:
        class: provider instanced
    """
    if config.PROVIDER not in providers:
        raise Exception(f'Provider {config.PROVIDER} is not available')
    provider = providers[config.PROVIDER]
    if service not in provider.__all__:
        raise Exception(
            'Service {} is not available '
            'for Provider {}'.format(service, config.PROVIDER)
        )
    return getattr(provider, service)()
